<?xml version="1.0"?>
<xsl:stylesheet
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:fo="http://www.w3.org/1999/XSL/Format"
   xmlns:xalan="http://xml.apache.org/xalan"
   version="1.0">
  <xsl:output method="xml" encoding="UTF-8" indent="yes" xalan:indent-amount="4"/>
  
  <xsl:template match="classes">
    <fo:root>
      <fo:layout-master-set>
        <fo:simple-page-master
                master-name="A4portrait"
                page-height="29.7cm"
                page-width="21cm"
                margin-top="2cm"
                margin-bottom="1cm"
                margin-left="2.5cm"
                margin-right="2.5cm">
          <fo:region-body margin-top="0cm"/>
          <fo:region-before extent="2cm"/>
          <fo:region-after extent="0cm"/>
        </fo:simple-page-master>
      </fo:layout-master-set>

      <fo:page-sequence
        master-reference="A4portrait"
        font-family="MS Gothic">
        <fo:static-content flow-name="xsl-region-after">
          <fo:block text-align="center" font-size="9pt">
            - <fo:page-number/> -
          </fo:block>
        </fo:static-content>

        <fo:flow flow-name="xsl-region-body">
          <fo:block font-size="18pt"
              line-height="24pt"
              space-after.optimum="15pt"
              text-align="left"
              padding-top="3pt">
            1. クラス＆メソッド一覧
          </fo:block>
      
          <xsl:apply-templates mode="summary"/>
        
          <fo:block font-size="18pt"
              line-height="24pt"
              space-after.optimum="15pt"
              text-align="left"
              padding-top="3pt"
              page-break-before="always">
            2. クラス詳細
          </fo:block>
          <xsl:apply-templates mode="detail"/>
        </fo:flow>
      </fo:page-sequence>
    </fo:root>
  </xsl:template>

  <xsl:template match="class" mode="summary">
    <fo:block font-size="12pt"
              line-height="14pt"
              space-after.optimum="3pt"
              text-align="justify">
      クラス <xsl:value-of select="name"/>
    
      <xsl:if test="count(methods/method) > 0">
        <fo:table margin-left="5mm">
          <fo:table-body>
            <xsl:apply-templates select="methods/method" mode="summary"/>
          </fo:table-body>
        </fo:table>
      </xsl:if>
    </fo:block>
  </xsl:template>
  
  <xsl:template match="method" mode="summary">
    <fo:table-row>
      <fo:table-cell text-indent="-1em" padding-left="1em">
        <fo:block><xsl:value-of select="signature"/></fo:block>
      </fo:table-cell>
      <fo:table-cell>
        <fo:block>
          <xsl:if test="contains(comment,'&#xA;')"><xsl:value-of select="substring-before(comment,'&#xA;')"/></xsl:if>
          <xsl:if test="not(contains(comment,'&#xA;'))"><xsl:value-of select="comment"/></xsl:if>
        </fo:block>
      </fo:table-cell>
    </fo:table-row>
  </xsl:template>
  
  <xsl:template match="class" mode="detail">
    <fo:block font-size="14pt"
              line-height="16pt"
              space-after.optimum="15pt"
              text-align="left"
              padding-top="3pt">
            2.<xsl:number format="1."/> <xsl:value-of select="name"/>
    </fo:block>

    <xsl:if test="string-length(comment) > 0">
      <fo:block margin-top="10px">
        <fo:block><xsl:text>【概要】</xsl:text></fo:block>
        <fo:block margin-left="10mm"><xsl:value-of select="comment"/></fo:block>
      </fo:block>
    </xsl:if>

    <xsl:apply-templates select="methods/method" mode="detail"/>
  </xsl:template>

  <xsl:template match="method" mode="detail">
    <fo:block background-color="#EEEEEE" margin-top="10pt" margin-bottom="10pt" padding="3pt">
      <xsl:value-of select="signature"/>
    </fo:block>
    
    <xsl:if test="count(parameters/parameter) > 0">
      <fo:block><xsl:text>【引数】</xsl:text></fo:block>
        <fo:table margin-left="10mm" table-layout="fixed">
          <fo:table-column column-width="20mm"/>
          <fo:table-column column-width="20mm"/>
          <fo:table-column/>
          <fo:table-header>
            <fo:table-row>
              <fo:table-cell margin="0" border="1px solid gray" text-align="center">
                <fo:block>型</fo:block>
              </fo:table-cell>
              <fo:table-cell margin="0" border="1px solid gray" text-align="center">
                <fo:block>名前</fo:block>
              </fo:table-cell>
              <fo:table-cell margin="0" border="1px solid gray" text-align="center">
                <fo:block>説明</fo:block>
              </fo:table-cell>
            </fo:table-row>
          </fo:table-header>
          <fo:table-body>
            <xsl:for-each select="parameters/parameter">
            <fo:table-row>
              <fo:table-cell margin-left="3px" border="1px solid gray">
                <fo:block margin="0"><xsl:value-of select="type/qualifiedName"/></fo:block>
              </fo:table-cell>
              <fo:table-cell margin-left="3px" border="1px solid gray">
                <fo:block margin="0"><xsl:value-of select="name"/></fo:block>
              </fo:table-cell>
              <fo:table-cell margin-left="3px" border="1px solid gray">
                <fo:block><xsl:value-of select="comment"/></fo:block>
              </fo:table-cell>
            </fo:table-row>
            </xsl:for-each>
          </fo:table-body>
        </fo:table>
    </xsl:if>
    
    <xsl:if test="string-length(comment) > 0">
      <fo:block margin-top="10px">
        <fo:block><xsl:text>【説明】</xsl:text></fo:block>
        <fo:block margin-left="10mm"><xsl:value-of select="comment"/></fo:block>
      </fo:block>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
